package edu.rice.comp211.laundry;

import edu.rice.comp211.laundry.garment.Pants;
import edu.rice.comp211.laundry.garment.Shirt;
import edu.rice.comp211.laundry.garment.Socks;
import edu.rice.comp211.laundry.io.IOProcess;
import edu.rice.comp211.laundry.io.TerminalIO;
import edu.rice.comp211.laundry.list.Predicate;
import edu.rice.comp211.laundry.student.Student;
import edu.rice.comp211.laundry.ui.SimLaundry2010Application;

import java.io.IOException;

public class Main {
   private static <T> T arraySearch(T[] a, Predicate<T> p) {
      for (T t : a) {
         if (p.test(t)) {
            return t;
         }
      }
      return null;
   }

   public static void main(String[] args) throws IOException {

      final String debugFlag = arraySearch(args, new Predicate<String>() {
         public boolean test(String s) {
            return s.equals("-d") || s.equals("-debug");
         }
      });
      final String graphicsFlag = arraySearch(args, new Predicate<String>() {
         public boolean test(String s) {
            return s.equals("-g") || s.equals("-graphics");
         }
      });

      /** fName non-null only if there is an argument not of the form "-*" */
      final String fName = arraySearch(args, new Predicate<String>() {
         public boolean test(String s) {
            return s.charAt(0) != '-';
         }
      });

      final Shirt shirt = new Shirt("white");
      final Pants pants = new Pants("white");
      final Socks socks = new Socks("white");
      final Student acker = new Student("J. H. Acker", shirt, pants, socks);

      if (graphicsFlag == null) {
         // Note; TerminalIO uses System.in if fName is null
         final IOProcess ioProc = new TerminalIO(fName);
         acker.simulate(ioProc, debugFlag != null);
      } else {
         SimLaundry2010Application.main(args);
      }
   }
}
