package edu.rice.comp211.laundry.list;

import edu.rice.comp211.laundry.iterator.IteratorI;

/**
 * A mutable linear list of elements of type T that can be scanned from first (front) to last (rear).  This interface
 * is easily implemented as a singly linked list.  This list always includes a dummy node preceding the first element
 * (following the last element) and behaves as if it is circular when scanned by an iterator.
 * @param <T> the type of elements held by this list 
 */
public interface ListI<T> extends EnumI<T> {
   /**
    * Factory method to instantiate a new empty list that can hold the same type of element.
    * @return a new empty list.
    */
   ListI<T> newList();

   /**
    * The number of elements in the list
    * @return the length of this list.
    */
   int length();

   /**
    * Inserts the object o at the front of this list
    * @param o the object to insert
    * @return this list .
    */
   ListI<T> insertFront(T o);

   /**
    * Inserts the object o at the rear of this list.
    * @param o the object to insert
    * @return this list .
    */
   ListI<T> insertRear(T o);

   /**
    * Removes the element at the front of the list.
    *
    * @return the removed element.
    */
   T remFront();

   /**
    * Factory method that instantiates a new iterator over this list.
    * @return The inherited newIterator method with a narrowed output type
    */
   IteratorI<T> newIterator();
}
