package edu.rice.comp211.laundry.student;

import edu.rice.comp211.laundry.garment.Garment;
import edu.rice.comp211.laundry.garment.Pants;
import edu.rice.comp211.laundry.garment.Shirt;
import edu.rice.comp211.laundry.garment.Socks;
import edu.rice.comp211.laundry.list.*;


/**
 * StudentEnvironment interface that the debugging code in nextCommand uses to access the current state of the
 * simulated student environment.
 */
public interface StudentEnvironment {

	/**
	 * The maximum number of garments that can be laundered at once.
	 */
	int MAX_LOAD = 15;

	/**
	 * Accessor for the student's name
	 * @return the name of the student
	 */
	String name();

	/**
	 * Accessor for the Shirt the student is currently wearing.
	 * @return the current shirt of the student
	 */
	Shirt shirt();

	/**
	 * Accessor for the Pants the student is currently wearing.
	 * @return the current pants of the student
	 */
	Pants pants();

	/**
	 * Accessor for the Socks the student is currently wearing.
	 * @return the current socks of the student
	 */
	Socks socks();

	/**
	 * Accessor for the student's pile of clean shirts
	 * @return the student's clean shirts pile
	 */
	EnumI<Garment> cleanShirts();  

	/**
	 * Accessor for the student's pile of clean pants
	 * @return the student's clean pants pile
	 */
	EnumI<Garment> cleanPants();  

	/**
	 * Accessor for the student's pile of clean socks
	 * @return the student's clean socks pile
	 */
	EnumI<Garment> cleanSocks();   

	/**
	 * Accessor for the student's pile of dirty garments
	 * @return the student's dirty clothes pile
	 */
	EnumI<Garment> dirtyPile();    

	/**
	 * Accessor for the student's laundry room (list of list of laundered garments)
	 * @return the student's laundry room
	 */
	EnumI<BiListI<Garment>> laundryRoom();  
}
