package edu.rice.comp211.laundry.sync;

import edu.rice.comp211.laundry.iterator.BiIteratorI;
import edu.rice.comp211.laundry.list.BiList;
import edu.rice.comp211.laundry.list.BiListI;

/**
 * Specialty BiList that is used by the GUI to create animations by slowing the speed of the list operations down.
 * Students should not need to use this class at all.
 */
public class SyncBiList<T> extends BiList<T> {
   @Override
   public synchronized boolean isEmpty() {
      final boolean result = super.isEmpty();
      gotoSleep();
      return result;
   }

   @Override
   public synchronized BiListI<T> newList() {
      final BiListI<T> result = super.newList();
      gotoSleep();
      return result;
   }

   @Override
   public synchronized int length() {
      final int result = super.length();
      gotoSleep();
      return result;
   }

   @Override
   public synchronized BiList<T> insertFront(final T o) {
      final BiList<T> result = super.insertFront(o);
      gotoSleep();
      return result;
   }

   @Override
   public synchronized BiList<T> insertRear(final T o) {
      final BiList<T> result = super.insertRear(o);
      gotoSleep();
      return result;
   }

   @Override
   public synchronized T remFront() {
      final T result = super.remFront();
      gotoSleep();
      return result;
   }

   @Override
   public synchronized BiIteratorI<T> newIterator() {
      final BiIteratorI<T> result = super.newIterator();
      gotoSleep();
      return result;
   }

   @Override
   public synchronized T remRear() {
      final T result = super.remRear();
      gotoSleep();
      return result;
   }

   private void gotoSleep() {
      try {
         Thread.sleep(((long)(Math.random() * 10000)) % 350);
      } catch (final InterruptedException e) {
         e.printStackTrace();
      }
   }
}
