package edu.rice.comp211.laundry.tests;

import edu.rice.comp211.laundry.garment.Garment;
import edu.rice.comp211.laundry.garment.Pants;
import edu.rice.comp211.laundry.garment.Shirt;
import edu.rice.comp211.laundry.garment.Socks;
import edu.rice.comp211.laundry.io.StringOutputStream;
import edu.rice.comp211.laundry.io.TerminalIO;
import edu.rice.comp211.laundry.list.BiList;
import edu.rice.comp211.laundry.list.BiListI;
import edu.rice.comp211.laundry.student.Student;
import junit.framework.TestCase;

import java.io.*;


public class LaundryTest extends TestCase {

   static final Shirt whShirt = new Shirt("white");
   static final Pants whPants = new Pants("white");
   static final Socks whSocks = new Socks("white");

   /**
    * Compares two files for equivalent text; missing EOL at EOF is ignored.
    */
   static void compareFiles(StringOutputStream os, File answer) throws IOException {
      BufferedReader testRdr = new BufferedReader(new StringReader(os.toString()));
      BufferedReader answerRdr = new BufferedReader(new FileReader(answer));
      compareReaders(testRdr, answerRdr);
   }

   static void compareFiles(File test, File answer) throws IOException {
      BufferedReader testRdr = new BufferedReader(new FileReader(test));
      BufferedReader answerRdr = new BufferedReader(new FileReader(answer));
      compareReaders(testRdr, answerRdr);
   }

   private static void compareReaders(BufferedReader testRdr, BufferedReader answerRdr)
         throws IOException {
      //int linectr = 0;
      for (int lineCtr = 0; true; lineCtr++) {
         String answerLine = answerRdr.readLine();
         String answerText = (answerLine == null) ? "" : answerLine;
         String testLine = testRdr.readLine();
         String testText = (testLine == null) ? "" : testLine;
         assertEquals("line [" + lineCtr + "] mismatch.", answerText, testText);
         if (testLine == null || answerLine == null) {
            break;
         }
      }
   }

   /**
    * A test method for the simulate method.
    */
   public void runTestSimulate(String prefix) throws IOException {
      String inName = prefix + "In.txt";
      String outName = prefix + "Out.txt";
      StringOutputStream temp = new StringOutputStream();

      TerminalIO ioProc = new TerminalIO(inName, temp);

      Student acker = new Student("James H. Acker", whShirt, whPants, whSocks);
      acker.simulate(ioProc, false);  // debugging off

      File out = new File(outName);
      compareFiles(temp, out);
   }

   public void test_Sample_file() throws IOException {
      runTestSimulate("sample");
   }

   public void test_Sys_file() throws IOException {
      runTestSimulate("Sys");
   }
   public void test_test_file() throws IOException {
      runTestSimulate("test");
   }

   private static InputStream convertToStream(String s) throws UnsupportedEncodingException {
      return new ByteArrayInputStream(s.getBytes("UTF-8"));
   }
   
   private static final String NL = System.getProperty("line.separator");

   public void test_Receive_cmds() throws IOException {
      InputStream is = convertToStream("receive Hawaiian shirt receive purple shirt change shirt");
      StringOutputStream os = new StringOutputStream();
      TerminalIO ioProc = new TerminalIO(is, os);
      Student acker = new Student("J. H. Acker", whShirt, whPants, whSocks);
      acker.simulate(ioProc, false);
      BiList<Garment> emptyGarments = new BiList<Garment>();  // DO NOT MUTATE THIS BiList
      Shirt hiShirt = new Shirt("Hawaiian");
      Shirt puShirt = new Shirt("purple");
      BiList<Garment> shirts = new BiList<Garment>().insertFront(hiShirt);
      BiList<Garment> dirtyPile = new BiList<Garment>().insertFront(whShirt);
      Student answer = new Student("J. H. Acker", puShirt, whPants, whSocks, shirts, emptyGarments, emptyGarments,
            dirtyPile, new BiList<BiListI<Garment>>());
      assertEquals("CorrectFinalState", answer, acker);
      /* NL is defined above to be the system-dependent line separator, which is a linefeed in Unix/Linux/Mac systems and CR+LF in Windows */
      String output = "received Hawaiian shirt"+NL+"received purple shirt"+NL+"doffed white shirt, donned purple shirt"+NL;
      assertEquals("CorrectStreamOutput", output, os.toString());
   }
}
